/*
 * Choeps Next Generation GUI
 * 
 * Brent Priddy <toopriddy@mailcity.com>
 *
 * Copyright(C) 1999, Adtran, Inc.
 * 
 * Distributed under the terms of the GNU General Public License (GPL) Version
 *
 * Functions for the gnome canvas and the page_object's that are on the canvas
 *
 */

#include <gnome.h>

#include "cheops-gui.h"
#include "gui-viewspace.h"
#include "gui-canvas.h"
#include "gui-utils.h"
#include "gui-settings.h"
#include "gui-config.h"
#include "gui-service.h"
#include "cheops-osscan.h"
#include "ip_utils.h"
#include "logger.h"
#include "gui-dns.h"
#include "gui-pixmap.h"

int page_object_event_dispatch(GtkObject *widget, GdkEvent *event, page_object *po);

extern int control_key;

void deleting_group(GtkWidget *w, page_object *po)
{
	po->group = NULL;
}

/*
 * Create a new page_abject with all fields avaliable for use
 */
page_object *page_object_new(GnomeCanvas *canvas)
{
	page_object *po;
	
	g_return_val_if_fail( canvas, NULL);
	g_return_val_if_fail( po = malloc(sizeof(page_object)), NULL );

	memset(po, 0, sizeof(page_object));

	po->flags = PAGE_OBJECT_CREATED_NEW;
	po->icon_file_name = makestring("unknown.xpm");
	po->canvas = canvas;
//	po->tooltips = gtk_tooltips_new();
//	gtk_tooltips_set_delay(po->tooltips, tooltips_timeout);
	
	po->group = gnome_canvas_item_new( gnome_canvas_root(canvas),
					  GNOME_TYPE_CANVAS_GROUP, NULL );
	po->box =   gnome_canvas_item_new( GNOME_CANVAS_GROUP(po->group),
					  GNOME_TYPE_CANVAS_RECT,
					  "fill_color", "white",
					  "outline_color", "black",
					  NULL );
	po->label = gnome_canvas_item_new( GNOME_CANVAS_GROUP(po->group),
					  GNOME_TYPE_CANVAS_TEXT,
					  "font", "fixed",
					  "fill_color", "black",
					  "anchor", GTK_ANCHOR_N,
					  NULL );
	po->icon  = gnome_canvas_item_new( GNOME_CANVAS_GROUP(po->group),
					  gnome_canvas_image_get_type(),
					  "anchor", GTK_ANCHOR_S,
					  NULL );

	// put a signal on the goup so we dont try to free it twice
	gtk_signal_connect(GTK_OBJECT(po->group), "destroy", deleting_group, po);

	gtk_signal_connect( GTK_OBJECT(po->group), "event", GTK_SIGNAL_FUNC(page_object_event_dispatch), po);

	return(po);
}

#define INITIAL_Y 50.0
#define INITIAL_X 40.0

char *page_object_get_name(page_object *po)
{
	char *name;
	if(po)
	{
		if(po->name == NULL)
			name = ip2str(po->ip);
		else
		{
			if(options_use_ip_for_label)
				name = ip2str(po->ip);
			else
				name = po->name;
		}
	}
	else
		name = "Unknown";
	return(name);
}	

/*
 * Display the page object, give it a pixmap, and put it on the canvas
 */
void page_object_display( page_object *po )
{
	double x1, x2, y1, y2;
	double nextx = 0.0;
	double nexty = INITIAL_Y;
	double boxheight, boxwidth;
	double nextydown = 0.0;
	char *name;
	
	g_return_if_fail( po );

	name = page_object_get_name(po);
	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->label), "text", name,NULL);
	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(po->label), &x1, &y1, &x2, &y2);

	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->box), "x1", x1-2.0, "x2", x2+2.0, "y1", y1-2.0, "y2", y2+2.0, NULL);

	if(po->im)
	{
		gdk_imlib_kill_image(po->im);
		po->im = NULL;
	}

	if(po->os_data && po->os_data->os)
	{
		if(po->icon_file_name)
			free(po->icon_file_name);
		
		po->icon_file_name = makestring(os_pixmap_list_get(po->os_data->os));	
	}

	po->im = get_my_imlib_image(po->icon_file_name);

	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->icon), "image", po->im, "width", (double)po->im->rgb_width, "height", (double)po->im->rgb_height, NULL);

//	gtk_tooltips_set_tip(po->tooltips, GTK_WIDGET(main_window->notebook), "this is a tip", NULL);
		
	if(po->flags & PAGE_OBJECT_CREATED_NEW)
	{
		gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(po->box), &x1, &y1, &x2, &y2);
		boxheight = y2 - y1;
		boxwidth = x2 - x1;
		gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(po->icon), &x1, &y1, &x2, &y2);
	
		if(po->next != NULL)
		{
			nextx = po->next->x + boxwidth;
			nexty = po->next->y;
			nextydown = boxheight + (y2 - y1);
		}
		po->x = INITIAL_X + nextx;
		po->y = nexty;
	
		if(po->x >= CANVAS_FIXED_WIDTH)
		{
			po->x = INITIAL_X;
			po->y += INITIAL_Y + nextydown;
		}
		if(po->y >= CANVAS_FIXED_HEIGHT)
		{
			po->x = INITIAL_X;
			po->y = INITIAL_Y;
		}
		po->flags &= ~PAGE_OBJECT_CREATED_NEW;

		gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->group), "x", (double)po->x, "y", (double)po->y, NULL); 	
	}
	if(po->flags & PAGE_OBJECT_CONFIG_NEW)
	{
		po->flags &= ~PAGE_OBJECT_CONFIG_NEW;
		gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->group), "x", po->x, "y", po->y, NULL); 	
	}
	
}

void page_object_highlight_highlight(page_object *po)
{
	po->flags |= PAGE_OBJECT_HIGHLIGHT;
	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->box), "fill_color", "blue", NULL);
	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->label), "fill_color", "white", NULL);
}

void page_object_highlight_unhighlight(page_object *po)
{
	static unsigned int unhighlight = ~PAGE_OBJECT_HIGHLIGHT;

	po->flags &= unhighlight; 
	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->box), "fill_color", "white", NULL);
	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->label), "fill_color", "black", NULL);
}	

/*
 * Highlight all or unhighlight all (i know it looks nasty but it is fast)
 */
void net_page_highlight_all(net_page *np, int highlight)
{
	page_object *po;
	
	if(highlight) /* highlight all */
	{
		for(po = np->page_objects;po;po = po->next)
		{
			if(!(po->flags & PAGE_OBJECT_HIGHLIGHT))
			{
				page_object_highlight_highlight(po);
			}
		}
	}
	else
	{
		for(po = np->page_objects;po;po = po->next)
		{
			if(po->flags & PAGE_OBJECT_HIGHLIGHT)
			{
				page_object_highlight_unhighlight(po);
			}
		}
	}
}

/*
 * Toggle the page_object's highlight state
 */
void page_object_highlight_toggle(page_object *po)
{
	if(po->flags & PAGE_OBJECT_HIGHLIGHT)
		page_object_highlight_highlight(po);
	else
		page_object_highlight_unhighlight(po);
}

/*
 * remove all plink's on the current net_page
 */
void net_page_unset_plink()
{
	net_page *np = get_current_net_page();
	
	np->plink = NULL;
}

/*
 * Set up the plink adding all highlighted items on the current net_page
 */
void net_page_set_plink()
{
	net_page *np = get_current_net_page();
	page_object *po;
	
	net_page_unset_plink();
	
	for(po = np->page_objects; po; po = po->next)
	{
		if(po->flags & PAGE_OBJECT_HIGHLIGHT)
		{
			po->plink = np->plink;
			np->plink = po;
		}
	}
}

/*
 * get a page object on the net_page
 */
page_object *page_object_get_pointer(net_page *np, char *name)
{
	page_object *p, *prev = NULL;
	
	for(p = np->page_objects; p; p = p->next)
	{
		if(!strcmp(name, p->name))
		{
			return(p);
		}
		prev = p;
	}
	return(NULL);
}


/*
 * delete a single page_object on the net_page's canvas
 */
void page_object_delete(net_page *np, page_object *po)
{
	page_object *p, *prev = NULL;
	os_stats *os;
	os_port_entry *port;
	
	for(p = np->page_objects; p; p = p->next)
	{
		if((void *)po == (void *)p)
		{
			if(p->group)
				gtk_object_destroy(GTK_OBJECT(p->group));
	
			if(p->data)	free(p->data);
			if(p->notes)	free(p->notes);
			if(p->name)	free(p->name);
			if(p->icon_file_name) 	free(p->icon_file_name);
			if(p->popup_list)	free(p->popup_list);
			
			os = p->os_data;
			if(os)
			{
				for(port = os->ports; port; port = os->ports)
				{
					if(port->name)
						free(port->name);
					os->ports = port->next;
					
					free(port);
				}
				free(os);
			}
			
			if(prev == NULL)
				np->page_objects = p->next;
			else
				prev->next = p->next;
			free(p);
			return;
		}
		prev = p;
	}
}

/*
 * delete all highlighted page_objects on a net_page (np can be NULL == use current page)
 */
void net_page_delete_page_object(net_page *np)
{
	page_object *po;
	page_object *next;
	
	if(np==0)
		np = get_current_net_page();
	
	for(po = np->plink; po; po = next)
	{
		next = po->plink;
		page_object_delete(np,po);
	}
	np->plink = NULL;
}			

/*
 * Save page_object notes and remove the notes window
 */
void page_object_hide_notes(GtkWidget *w,page_object *po)
{
	char *new = gtk_editable_get_chars(GTK_EDITABLE(po->notes_widget), 0, -1);
	
	if(po->notes == NULL || strcmp(po->notes,new))
	{
		if(po->notes)
			g_free(po->notes);
		po->notes = new;
	}
	else
	{
		g_free(new);
	}
	
	gtk_widget_destroy(po->notes_window);
}

/*
 * Display the notes window and add any text in it that is stored in the page object
 */
void page_object_show_notes(GtkWidget *w, gpointer data)
{
	net_page *np = get_current_net_page();
	page_object *po = np->plink;
	
	if(po)
	{
		GtkWidget *button;
		GtkWidget *vbox;
		GtkWidget *window;
		GtkWidget *edit;
		char buf[256];
		
		strcpy(buf, "Notes for ");
		strcat(buf, page_object_get_name(po));
		
		window = gtk_window_new(GTK_WINDOW_DIALOG);
		gtk_window_set_title(GTK_WINDOW(window),buf);
		gtk_widget_set_usize(window,200,200);
		gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
		
		vbox = gtk_vbox_new(FALSE, 5);
		
		edit = gtk_text_new(NULL,NULL);
		gtk_text_set_editable(GTK_TEXT(edit),TRUE);
		gtk_box_pack_start(GTK_BOX(vbox), edit, TRUE, TRUE, 0);
		
		button = gtk_button_new_with_label("close");
		gtk_signal_connect(GTK_OBJECT(button), "clicked", page_object_hide_notes, po);
		gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
		
		gtk_container_add(GTK_CONTAINER(window),vbox);
		
		if(po->notes)
			gtk_text_insert(GTK_TEXT(edit),NULL,NULL,NULL,po->notes,-1);
		
		po->notes_widget = edit;
		po->notes_window = window;
		
		gtk_widget_show(vbox);
		gtk_widget_show(button);
		gtk_widget_show(edit);
		gtk_widget_show(window);
	}
}

/*
 * Save page_object info and remove the info window
 */
void page_object_hide_info(GtkWidget *w,page_object *po)
{
	gtk_widget_destroy(po->info_window);
}

/*
 * Display the Info about the page object
 */
void page_object_show_info(GtkWidget *w, gpointer data)
{
	net_page *np = get_current_net_page();
	page_object *po = np->plink;
	os_port_entry *port;
	GtkWidget *button;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *window;
	GtkWidget *label;
	GtkWidget *clist;
	GtkWidget *scrolled_window;
	char buf[256];
	char *clist_titles[] = { "Service", "Port", "Protocol" };
	char *bufs[ sizeof(clist_titles)/sizeof(char *) ];
	char port_buf[20];
	int i = 0;
	
	if(po)
	{
		strcpy(buf, "Info for ");
		strcat(buf, page_object_get_name(po));
		
		window = gtk_window_new(GTK_WINDOW_DIALOG);
		gtk_window_set_title(GTK_WINDOW(window),buf);
//		gtk_widget_set_usize(window,200,200);
		gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
		
		vbox = gtk_vbox_new(FALSE, 5);

/* Operating system */
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_widget_show(hbox);
		label = gtk_label_new("Operating System: ");
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
		gtk_widget_show(label);
		if(po->os_data)
			sprintf(buf, "%s", po->os_data->os);
		else
			sprintf(buf, "Not known");
		label = gtk_label_new(buf);
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
		gtk_widget_show(hbox);

/* IP Address */		
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_widget_show(hbox);
		label = gtk_label_new("IP Address: ");
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
		label = gtk_label_new( ip2str(po->ip) );		
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);

/* DNS Address */
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_widget_show(hbox);
		label = gtk_label_new("DNS Address: ");
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
		if(po->name)
			label = gtk_label_new( po->name );
		else
			label = gtk_label_new( "Not Known" );
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);

/* Services running */
		clist = gtk_clist_new_with_titles(sizeof(clist_titles)/sizeof(char*), clist_titles);
		gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
		gtk_clist_set_shadow_type(GTK_CLIST(clist), GTK_SHADOW_ETCHED_OUT); 
		gtk_clist_set_column_width(GTK_CLIST(clist), 0, 120);
		gtk_clist_set_column_width(GTK_CLIST(clist), 1, 70);
		gtk_clist_set_column_width(GTK_CLIST(clist), 0, 60);
		
		
		if(po->os_data)
		{
			for(port = po->os_data->ports; port; port = port->next)
			{
				bufs[0] = (port->name + 3);
				
				sprintf(port_buf, "%d", port->port);
				bufs[1] = port_buf;
				
				bufs[2] = ( port->protocol == 6 ? "TCP":"UDP");
				gtk_clist_prepend(GTK_CLIST(clist), bufs);
				i++;
			}
		}
		if(i > 15)
		{
			scrolled_window = gtk_scrolled_window_new(NULL,NULL);
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), clist);
			gtk_widget_set_usize(window,350,600);
			gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, FALSE, FALSE, 5);
			gtk_widget_show(scrolled_window);
			gtk_widget_show(clist);
		}
		else
		{
			if(i != 0)
			{
				gtk_box_pack_start(GTK_BOX(vbox), clist, FALSE, FALSE, 5);
				gtk_widget_show(clist);
			}
		}
		
		button = gtk_button_new_with_label("close");
		gtk_signal_connect(GTK_OBJECT(button), "clicked", page_object_hide_info, po);
		gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
		
		gtk_container_add(GTK_CONTAINER(window),vbox);
		
		po->info_window = window;
		
		gtk_widget_show(button);
		gtk_widget_show(vbox);
		gtk_widget_show(window);
	}
}


void page_object_service_callback(GtkWidget *widget, gpointer data)
{
	int port = (u32)data >> 8;
	int protocol = (u32)data & 0xFF;

	// there can only be one page_object selected that can show the service list,
	// so i use this quick and dirty way to get at the IP address :)
	service_callback( (get_current_net_page())->plink->ip,port, protocol);
}

/*
 * Shows an option list for things to do on a oage_object
 */
void page_object_show_option_list(net_page *np, page_object *po, GdkEventButton *eb)
{
	GtkItemFactory *itemf;
	GtkWidget *pagemenu;
	int nOptions;
	os_port_entry *port;
	GtkItemFactoryEntry *entry;
	char buf[200];
	char *service;
	static GtkItemFactoryEntry options[] = 
	{
		{ "/p/Delete",	NULL,	do_delete_page_object,	0 },
		{ "/p/sep1",	NULL,	NULL,			0,	"<Separator>" },
		{ "/p/notes",	NULL,	page_object_show_notes,	0 },
		{ "/p/info",	NULL,	page_object_show_info,	0 },
		{ "/p/sep2",	NULL,	NULL,			0,	"<Separator>" }
	};
	
	itemf = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<blah>",NULL);

	po->flags &= ~PAGE_OBJECT_CHANGE_POPUP_LIST;
	if(po->popup_list)
		free(po->popup_list);

	nOptions = sizeof(options)/sizeof(options[0]);
	
	// if there are multiple obgects selected then dont display ports
	if( np->plink && !np->plink->plink )
	{
		if(po->os_data)
			for(port = po->os_data->ports; port; port = port->next)
				nOptions++;
	}
	else
		nOptions -= 3;

	/* record for a later date */
	po->popup_list_number = nOptions;
	po->popup_list = malloc(sizeof(GtkItemFactoryEntry)*nOptions);
	memcpy(po->popup_list,options,po->popup_list_number*sizeof(GtkItemFactoryEntry));

	if( np->plink && !np->plink->plink )
	{
		entry = po->popup_list;
		entry += sizeof(options)/sizeof(options[0]);

		if(po->os_data)
		{
			for(port = po->os_data->ports; port; port = port->next)
			{
				service = get_service(port->port, port->protocol);
				if(service && service[0] != '\0')
					sprintf(buf,"/p/%s",service);
				else
					sprintf(buf,"/p/Port %d", port->port);
					
				port->name = makestring(buf);
				entry->path = port->name;
				entry->accelerator = NULL;
				entry->callback = page_object_service_callback;
				entry->callback_action = (port->port << 8) + port->protocol;
				entry->item_type = NULL;
			
				entry++;
			}		
		}
	}
	gtk_item_factory_create_items(itemf, nOptions, po->popup_list, NULL);
		
	pagemenu = gtk_item_factory_get_widget(itemf, "/p");
	gtk_menu_popup(GTK_MENU(pagemenu), NULL, NULL, NULL, NULL, eb->button, eb->time);
}

/*
 * event dispatcher for the page_object
 */
int page_object_event_dispatch(GtkObject *widget, GdkEvent *event, page_object *po)
{
	double x,y;
	GdkEventMotion *em = (GdkEventMotion *)event;
	GdkEventButton *eb = (GdkEventButton *)event;
	page_object *pl;
	net_page *np;
	
	switch( event->type )
	{
		case GDK_BUTTON_PRESS:
			if(eb->button == 1)
			{
				po->x = eb->x;
				po->y = eb->y;

				gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(po->group));

				po->flags |= PAGE_OBJECT_GRAB;
				
				if(eb->state & GDK_CONTROL_MASK)
				{
				/* trying to select multiple objects */
					if(po->flags & PAGE_OBJECT_HIGHLIGHT)
					{
					/* selected one that is already highlighted, unhighlight it */
						page_object_highlight_unhighlight(po);
						po->flags &= ~PAGE_OBJECT_GRAB;
					}
					else
					{
					/* it is not highlighted, so highlight it */
						page_object_highlight_highlight(po);
					}
				}
				else
				{
				/* not selecting multiple objects */
					if(!(po->flags & PAGE_OBJECT_HIGHLIGHT))
					{
					/* the object is not highlighted, unhighlight all and then highlight the one selected */
						np = get_current_net_page();
						net_page_highlight_all(np, FALSE);
						page_object_highlight_highlight(po);
						net_page_unset_plink();
					}
					page_object_highlight_highlight(po);
				}
				
				net_page_set_plink();
			}
			else
			{
				if(eb->button == 3)
				{
					if(eb->state & GDK_CONTROL_MASK)
					{
						np = get_current_net_page();
						page_object_highlight_highlight(po);
						net_page_set_plink();
						while(gtk_events_pending())
							gtk_main_iteration();
						page_object_show_option_list(np,po,eb);
					}
					else
					{
						np = get_current_net_page();
						if(!(po->flags & PAGE_OBJECT_HIGHLIGHT))
						{
							net_page_highlight_all(np, FALSE);
							page_object_highlight_highlight(po);
							net_page_set_plink();
						}
						while(gtk_events_pending())
							gtk_main_iteration();
						page_object_show_option_list(np,po,eb);
					}
					po->flags &= ~PAGE_OBJECT_GRAB;
				}
			}
			break;
			
		case GDK_BUTTON_RELEASE:
			po->flags &= ~PAGE_OBJECT_GRAB;
			break;
		
		case GDK_MOTION_NOTIFY:
			if(po->flags & PAGE_OBJECT_GRAB)
			{
				np = get_current_net_page();
				
				x = (double)em->x;
				y = (double)em->y;
					
				if(x < 0) x = 0;
				if(y < 0) y = 0;
				
				x = x - po->x;
				y = y - po->y;
					
				for(pl = np->plink; pl; pl = pl->plink)
				{
					if( ((pl->x + x) >= 0) && ((pl->x + x) <= CANVAS_FIXED_WIDTH) )
						pl->x += x;
					else
						x = 0;
					
					if( ((pl->y + y) >= 0) && ((pl->y + y) <= CANVAS_FIXED_HEIGHT) )
						pl->y += y;
					else
						y = 0;
						
					gnome_canvas_item_move(GNOME_CANVAS_ITEM(pl->group), (double)x, (double)y);
	//				printf("x=%f y=%f\n",pl->x, pl->y);
				}
			}
			break;
		
		default:
			break;
	}
	return(0);
}	

/*
 * event function for the canvas (tidbit: the canvas event is called before the page items)
 */
int net_page_event_dispatch(GtkObject *widget, GdkEvent *event, net_page *np)
{
	GdkEventButton *eb = (GdkEventButton *)event;
	GdkEventMotion *em = (GdkEventMotion *)event;
	page_object *po;

	switch( event->type )
	{
		case GDK_BUTTON_PRESS:
			if(eb->button == 1)
			{
				if( !(eb->state & GDK_CONTROL_MASK) )
				{
					if(NULL == gnome_canvas_get_item_at(GNOME_CANVAS(np->canvas),eb->x,eb->y) )
					{
					/* we just hit the page to get rid of the highlight */
						net_page_highlight_all(np, FALSE);
						net_page_unset_plink();
						np->button_state |= GDK_BUTTON_PRESS;
						np->select_x = eb->x;
						np->select_y = eb->y;
					}
				}
			}
			else
			{
				if(NULL == gnome_canvas_get_item_at(GNOME_CANVAS(np->canvas),eb->x,eb->y) )
				{
					if (eb->button == 3)
					{
						GtkWidget *pagemenu = gtk_item_factory_get_widget(main_window->itemf, "/Viewspace");
						gtk_menu_popup(GTK_MENU(pagemenu), NULL, NULL, NULL, NULL, eb->button, eb->time);
					}
				}
			}
			break;
			
		case GDK_BUTTON_RELEASE:
			np->button_state &= ~GDK_BUTTON_PRESS;
			if(np->select_rectangle)
			{
				gtk_object_destroy(GTK_OBJECT(np->select_rectangle));
				np->select_rectangle = NULL;
			}
			break;
		
		case GDK_MOTION_NOTIFY:
			if(np->button_state & GDK_BUTTON_PRESS)
			{
				double x,y,x1,x2,y1,y2;
				if(np->select_rectangle == NULL)
				{
					np->select_rectangle = gnome_canvas_item_new(	gnome_canvas_root(GNOME_CANVAS(np->canvas)),
											GNOME_TYPE_CANVAS_RECT,
											"outline_color", "black",
											NULL);
					gnome_canvas_item_move(  GNOME_CANVAS_ITEM(np->select_rectangle), 
								(double)np->select_x,
								(double)np->select_y);
				}
				x = em->x - np->select_x;
				y = em->y - np->select_y;
				
				x1 = (double)(x < 0 ? x : 0 );
				y1 = (double)(y > 0 ? 0 : y);
				x2 = (double)(x < 0 ? 0 : x );
				y2 = (double)(y < 0 ? 0 : y);
								
				gnome_canvas_item_set(  GNOME_CANVAS_ITEM(np->select_rectangle), 
							"x1", x1,
							"y1", y1,
							"x2", x2,
							"y2", y2,
							NULL); 
				x1 += np->select_x;
				y1 += np->select_y;
				x2 += np->select_x;
				y2 += np->select_y;
							
//				printf("x1=%f y1=%f x2=%f y2=%f\n",x1,y1,x2,y2);
				 
				for(po = np->page_objects; po; po = po->next)
				{
					int pox = po->x;
					int poy = po->y;
					if( (pox > x1) && (pox < x2) && (poy > y1) && (poy < y2) )
					{
						if( !(po->flags & PAGE_OBJECT_HIGHLIGHT) )
							page_object_highlight_highlight(po);
					}
					else
					{
						if( (po->flags & PAGE_OBJECT_HIGHLIGHT) )
							page_object_highlight_unhighlight(po);
					}
				}
				while(gtk_events_pending())            /* show something so it does not look that choppy */
					gtk_main_iteration();
				net_page_set_plink();
			
			}
			break;

		default:
			break;
	}
	return(0);
}	

